require 'logger'
require_relative 'plm_rt'

module PLM
  module Model
    class Error
      module ClassMixin
        def send_error(params={})
          e       = params.fetch(:error, StandardError.new("No reason issued"))
          session = params.fetch(:session,{})
          begin
            error_manager = PLM::Model::Config::CONFIG[:ticket]
            error_manager[:type] = 'ticket'
          rescue
            error_manager = {type: 'stderr'}
          end
          case error_manager[:type]
          when 'ticket'
            send_ticket config: error_manager,
            message: e.message,
            reason:  e.class.name,
            session: session
          when 'stderr'
            logger message: e.message,
            reason:       e.class.name,
            session:      session
          else
            puts "No manager set: Reason: #{e.class.name} Message: #{e.message} for #{session}"
          end
        end

        def send_ticket(params={})
          ticket_config = params.fetch(:config, PLM::Model::Config::CONFIG[:ticket])
          reason        = params.fetch(:reason,'PLM::Model::Error')
          message       = params.fetch(:message ,'No reason issued')
          session       = params.fetch(:session,{})

          credentials = ticket_config[:credentials]
          credentials[:token]=credentials[:password] if credentials.has_key?(:password)
          credentials[:token]=credentials[:token] if credentials.has_key?(:token)
          rt = PLM_RT.new(credentials)

          message = message+"\npour une connexion venant de :\n#{session}" unless session == {}
          tickets = rt.query(query: "Subject LIKE \"#{reason}\" and (Status = 'new' or Status = 'open')")
          if tickets["count"] == 0
            attrs = {
              Subject: reason,
              Queue:  ticket_config[:queue],
              #'Owner'      => ticket_config[:owner],
              Content:   message
            }

            attrs[:Cc] = params[:Cc] if params.has_key?(:Cc)

            rt.create(attrs)
          else

            attrs = {
              id: tickets["items"].last['id'],
              Content: message
            }

            rt.comment(attrs)
          end

        end

        def resolve_ticket(params={})
          ticket_config = params.fetch(:config, PLM::Model::Config::CONFIG[:ticket])
          reason        = params.fetch(:reason,'PLM::Model::Error')
          message       = params.fetch(:message ,'')

          credentials = ticket_config[:credentials]
          credentials[:token]=credentials[:password] if credentials.has_key?(:password)
          credentials[:token]=credentials[:token] if credentials.has_key?(:token)
          rt = PLM_RT.new(credentials)

          tickets = rt.query(query: "Subject LIKE \"#{reason}\" and (Status = 'new' or Status = 'open')")
          tickets.items.each do |ticket|

            id = ticket['id']

            rt.comment({ id: id, Content: message }) unless message == ''

            rt.edit(id: id, Status: "resolved")
          end
        end

        def logger(params={})
          reason  = params.fetch(:reason,'PLM::Model::Error')
          message = params.fetch(:message ,'No reason issued')
          session = params.fetch(:session,{})

          message = message+"\npour une connexion venant de :\n#{session}" unless session == {}
          logger  = Logger.new(STDERR)

          logger.error "Reason: #{reason} Message: #{message}"
          logger.close
        end
      end
    end
  end
end
