module PLM
  module Model

    GUEST = 111

    class Identity
      def initialize(session)
        @convergence_data = session
      end
      #
      # Retourne vrai si la session n'est pas anonyme
      #
      def known?
        !@convergence_data.nil?
      end
      #
      # Applique un filtre générique à l'identité
      # Pour une entree de type : @convergence_data={"STATUS"=>"plm","FROM"=>"IMB"}
      # puts apply_filter => true
      # puts apply_filter({}) => true
      # puts apply_filter({"STATUS"=>"plm"}) => true
      # puts apply_filter({"STATUS"=>"plm","FROM"=>"IMB"}) => true
      # puts apply_filter({"STATUS"=>["plm","toto"],"FROM"=>"IMB"}) => true
      # puts apply_filter({"STATUS"=>["plm","toto"],"FROM"=>["IMB","la"]}) => true
      # puts apply_filter({"STATUS"=>"plm","FROM"=>"la"}) => false
      # puts apply_filter({"STATUS"=>"plm","FRAM"=>"la"}) => false
      # puts apply_filter({"STATUS"=>"toto"}) => false
      # puts apply_filter({"bal"=>"3"}) => false
      #
      #
      def apply_filter(params={})
        result = true
        params.each do |key, values|
          if @convergence_data[key].nil?
            return false
          else
            values = Array(values) unless values.is_a?(Array)
            result = values.member? @convergence_data[key]
          end
        end
        result
      end
      #
      # Retourne l'IDP si authentification
      #
      def idp
        @convergence_data['IDP'] if known?
      end
      #
      # Retourne vrai si la convergence a détecté un compte PLM
      #
      def user?
        known? && !@convergence_data['USER'].nil?
      end
      #
      # Retourne dans ce cas le compte PLM en question
      #
      def user
        @convergence_data['USER'] if user?
      end
      #
      # Retourne les informations personnelles
      #
      def names
        a={}
        if known?
          %w[SN CN displayName givenName].each do |name|
            a[name] = @convergence_data[name].nil? ? '' : @convergence_data[name]
          end
        end
        a
      end
      #
      # Retourne l'email associé à mon identité de connexion
      #
      def mail(which=nil)
        case which
        when 'contact'
          @convergence_data['mailLAB'] if user?
        when 'mathrice'
          @convergence_data['mailPLM'] if user? && !@convergence_data['mailPLM'].nil?
        when 'all'
          @convergence_data['MAILS']
        else
          @convergence_data['MAIL']
        end
      end
      #
      # Retourne les emails avec le type
      #
      def mails
        a = {}
        if known?
          a[:contact]     = @convergence_data['mailLAB'].downcase if user?
          a[:mathrice]    = @convergence_data['mailPLM'] if user? && @convergence_data['mailPLM']
          a[:altMathrice] = @convergence_data['mailAPLM'].map(&:downcase) if user? && @convergence_data['mailAPLM']
          a[:all]         = @convergence_data['MAILS']
          a[:emath]       = @convergence_data['emathMAILS'].map(&:downcase) if @convergence_data['emathMAILS']
          a[:idp]         = @convergence_data['MAILI']
        end
        a
      end
      #
      # Retourne les ID Mathdoc des organismes dans emath si insmi?
      #
      def emath_entities
        @convergence_data['LAB'] if known? && !@convergence_data['LAB'].nil?
      end
      # Pour compatibilité (obsolète)
      def labs
        emath_entities
      end
      #
      # Retourne vrai si la convergence a permi d'identifier l'appartenance
      # à une branche PLM (permet la gestion des droits des revues par exemple)
      #
      def plm_entity?(myou=nil)
        if known? && !@convergence_data['OU'].nil?
          unless myou
            @convergence_data['OU'].count > 0
          else
            myou=[myou] unless myou.is_a?(Array)
            (@convergence_data['OU'] & myou.map(&:to_s)).count > 0
          end
        else
          return false
        end
      end
      # Pour compatibilité (obsolète)
      def ou?(myou=nil)
        plm_entity?(myou)
      end
      #
      # Retourne vrai si c'est un compte extérieur (même invité dans une branche)
      #
      def plm_guest?
        plm_entity?(GUEST)
      end
      #
      # Retourne dans le cas la liste des labos PLM d'appartenance à cette personne
      #
      def plm_entities
        @convergence_data['OU'] if plm_entity?
      end
      # Pour compatibilité (obsolète)
      def ou
        plm_entities
      end
      #
      # Je suis gestionnaire de labo ou non (dans l'absolu ou pour un labo donné)
      #
      def admin?(ou=nil)
        if known? && !@convergence_data['ADMIN'].nil?
          unless ou
            @convergence_data['ADMIN'].count > 0
          else
            ou=[ou] unless ou.is_a?(Array)
            (@convergence_data['ADMIN'] & ou.map(&:to_s)).count > 0
          end
        else
          return false
        end
      end
      #
      # Retourne les labos dont je suis gestionnaire PLM
      #
      def admin
        @convergence_data['ADMIN'] if admin?
      end
      #
      # Retourne vrai si je suis emath dans un labo INSMI sans compte PLM
      #
      def converged_from_insmi?
        @convergence_data['STATUS'] == 'insmi'
      end
      #
      # Retourne vrai si je suis emath dans un labo INSMI et détection de mon compte PLM
      #
      def converged_from_insmi_with_plm?
        converged_from_insmi? &&
          @convergence_data['hasMailInPLMAccounts']
      end
      #
      # Retourne vrai si j'apparais dans l'annuaire de la communauté
      #
      def converged_from_emath?
        @convergence_data['FROM']   =~ /.*emath$/
      end
      #
      # les deux précédement vrais
      #
      def converged?
        known? && (converged_from_insmi? || converged_from_emath?)
      end
      #
      # Je n'ai pas de compte PLM je n'apparais pas dans l'annuaire emath
      #
      def other?
        known? && @convergence_data['STATUS'] == 'other'
      end
      #
      #
      #
      def classification
      end
      #
      #
      #
      def classification=(_classification)
      end
      #
      #
      #
      def civility

      end
      #
      #
      #
      def civility=(_civility)
      end

      #
      # Gestion des levees d'exceptions
      #
      module Error
        class IdentityUnknown < StandardError; end
        class IdentityError < StandardError; end
        class AdminError < StandardError; end
      end

      def known!
        raise Error::IdentityUnknown, "Anonymous session not allowed !" unless known?
      end

      def user!
        raise Error::IdentityError, "PLM User unknown !" unless user?
      end

      def admin!
        raise Error::AdminError, "You are not PLM administrator" unless admin?
      end
    end
    #
    #
    #
  end
end
